# This configuration will be added by the Rails-Application
config = Rails.application.config.services[:OCR]
OCR_SERVICE = config[:OCR_SERVICE]
OCR_PROJECT_PATH = config[:OCR_PROJECT_PATH]

# Decorator to add some ocr features to a document
Nukleus::Document.class_eval do
    def validate 
        request_ocr_data
    end

    private

    def request_ocr_data
        local_temp_file = self.document_file.download
        resp = ocr_service_connection.put(OCR_PROJECT_PATH) do  |req| 
            req.headers['Content-Type'] = self.document_file_content_type
            req.headers['Content-Length'] = local_temp_file.size.to_s
            req.params["customer"] = "Kofax"
            req.body = Faraday::UploadIO.new(local_temp_file.path, self.document_file_content_type)
        end
        # Delete local temp file. Otherwise we will mess up our local file system
        local_temp_file.delete

        Rails.logger.info "OCR-Response status: #{resp.status}"
        # We always want to trigger the next job. Thus, we have to save some meta-data,
        # even if we don't have them.
        hash_to_add = nil
        if resp.status == 200
            if resp.present? && resp.body.present?
                hash_to_add = { ocr_data: resp.body }
            end
        else
            # Error: Save error response as meta-data
            Rails.logger.error "OCR-Response body: #{resp.body}"
            hash_to_add = { ocr_error: resp.body }
        end
        unless hash_to_add.nil?
            self.meta_data = hash_to_add
        end
        self.save
    end

    def ocr_service_connection
        Faraday.new(:url => OCR_SERVICE) do |f|
            f.request  :multipart             
            f.response :logger         # log requests to STDOUT
            f.adapter  Faraday.default_adapter  # make requests with Net::HTTP
            f.use FaradayMiddleware::ParseJson, content_type: 'application/json'
        end
    end

end